//NOTE This is a .h file but we expect it to be included exactly one (in main.cpp).

#include <Adafruit_MPR121.h>

constexpr uint8_t TOUCH_I2C_ADDRESS[2] = { 0x5a, 0x5c };

struct HeizungTouch {
  Adafruit_MPR121 device[2];
  bool available[2];
  uint16_t touched[2], touchedPrev[2];

  void init() {
    for (int i=0;i<2;i++) {
      touched[i] = 0;
      touchedPrev[i] = 0;

      available[i] = device[i].begin(TOUCH_I2C_ADDRESS[i]);
      if (!available[i])
        continue;
      auto& cap = device[i];

      // disable it, again
      cap.writeRegister(MPR121_ECR, 0x0);

      // auto-config code (mostly) copied from Adafruit library
      cap.writeRegister(MPR121_AUTOCONFIG0, 0x0B);

      // correct values for Vdd = 3.3V
      cap.writeRegister(MPR121_UPLIMIT, 200);     // ((Vdd - 0.7)/Vdd) * 256
      cap.writeRegister(MPR121_TARGETLIMIT, 180); // UPLIMIT * 0.9
      cap.writeRegister(MPR121_LOWLIMIT, 130);    // UPLIMIT * 0.65

      // enable X electrodes and start MPR121
      // enable virtual proximity electrode, as well
      cap.writeRegister(MPR121_ECR, 0x8c | 0x30);
    }
  }

  void update(unsigned long now) {
    for (int i=0;i<2;i++) {
      touchedPrev[i] = touched[i];
      touched[i] = available[i] ? device[i].touched() : 0;
    }
  }

  bool hasChanged() {
    bool changed = false;
    for (int i=0;i<2;i++) {
      changed |= (touched[i] != touchedPrev[i]);
    }
    return changed;
  }
};
